#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2017 Luke Hinds <lukehinds@gmail.com>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0
##############################################################################

"""
    Accepts the --patchset argument and iterates through each line of the
    patchset file to perform various checks such as if the file is a binary, or
    contains a blacklisted string. If any violations are found, the script
    exits with code 1 and logs the violation(s) found.
"""

from __future__ import division, print_function, absolute_import
import ipaddress
import json
import logging
import hashlib
import six.moves.configparser
import sys
import os
import re
import time
from binaryornot.check import is_binary
from . import get_lists
from . import virus_total

logger = logging.getLogger(__name__)
config = six.moves.configparser.SafeConfigParser()
config.read('anteater.conf')
anteater_files = config.get('config', 'anteater_files')
reports_dir = config.get('config', 'reports_dir')

failure = False
hasher = hashlib.sha256()

def prepare_patchset(project, patchset,  bincheck, ips, urls):
    """ Create black/white lists and default / project waivers
        and iterates over patchset file """
    # Get Various Lists / Project Waivers
    lists = get_lists.GetLists()

    # Get file name black list and project waivers
    file_audit_list, file_audit_project_list = lists.file_audit_list(project)

    # Get file content black list and project waivers
    flag_list, ignore_list = lists.file_content_list(project)

    # Get URL Ignore Lists
    url_ignore = lists.url_ignore(project)

    # Get URL Ignore Lists
    ip_ignore = lists.ip_ignore(project)

    # Get File Ignore Lists
    file_ignore = lists.file_ignore()

    ignore_directories = lists.ignore_directories(project)

    if bincheck or ips or urls:
        try:  
            apikey = os.environ["VT_KEY"]
        except KeyError: 
            logger.error("Please set your virustotal.com API key as an environment variable")
            sys.exit(1)
        try:
            vt_rate_type = config.get('config', 'vt_rate_type')
        except six.moves.configparser.NoSectionError:
            logger.error("A config section is required for vt_rate_type with a public | private option ")
            sys.exit(1)

        patten = re.compile(r'\bpublic\b|\bprivate\b')
        if not patten.match(vt_rate_type):
            logger.error("Unrecognized %s option for vt_rate_type", vt_rate_type)
            sys.exit(1)

    # Open patch set to get file list
    try:
        fo = open(patchset, 'r')
        lines = fo.readlines()
    except IOError:
        logger.error('%s does not exist', patchset)
        sys.exit(1)

    for line in lines:
        patch_file = line.strip('\n')
        # Perform binary and file / content checks
        scan_patch(project, patch_file, bincheck, ips, urls, file_audit_list,
                   file_audit_project_list, flag_list, ignore_list,
                   file_ignore, ignore_directories, url_ignore, ip_ignore, apikey)

    # Process final result
    process_failure()


def scan_patch(project, patch_file, bincheck, ips, urls, file_audit_list,
               file_audit_project_list, flag_list, ignore_list, file_ignore,
               ignore_directories, url_ignore, ip_ignore, apikey):

    """ 
    Scan actions for each commited file in patch set
    """
    global failure
    split_path = patch_file.split(project + '/', 1)[-1]

    if not any(x in split_path for x in ignore_directories):
        if is_binary(patch_file) and bincheck:
            hashlist = get_lists.GetLists()
            binary_hash = hashlist.binary_hash(project, split_path)
            with open(patch_file, 'rb') as afile:
                buf = afile.read()
                hasher.update(buf)
                sha256hash = hasher.hexdigest()
                
            if sha256hash in binary_hash:
                logger.info('Found matching file hash for: %s',
                            patch_file)
            else:
                logger.error('Non Whitelisted Binary file: %s',
                             patch_file)
                
                scan_binary(patch_file, project, sha256hash, apikey) 
                # logger.error('Submit patch with the following hash: %s',
                failure = True
            with open(reports_dir + "binaries-" + project + ".log", "a") \
                    as gate_report:
                gate_report.write('Non Whitelisted Binary file: {0}\n'.
                                  format(patch_file))
        else:
            # Check file names / extensions
            if file_audit_list.search(patch_file) and not \
                    file_audit_project_list.search(patch_file):
                match = file_audit_list.search(patch_file)
                logger.error('Blacklisted file: %s', patch_file)
                logger.error('Matched String: %s', match.group())
                failure = True
                with open(reports_dir + "file-names_" + project + ".log", "a") \
                        as gate_report:
                    gate_report.write('Blacklisted file: {0}\n'.
                                      format(patch_file))
                    gate_report.write('Matched String: {0}'.
                                      format(match.group()))

            # Open file to check for blacklisted content
            if not is_binary(patch_file):
                try:
                    fo = open(patch_file, 'r')
                    lines = fo.readlines()
                    file_exists = True
                except IOError:
                    file_exists = False

                if file_exists and not patch_file.endswith(tuple(file_ignore)):
                    for line in lines:
                        # Find IP Addresses and send for report to Virus Total
                        if ips:
                            ipaddr = re.findall(r'(?:\d{1,3}\.)+(?:\d{1,3})', line)
                            if ipaddr:
                                ipaddr = ipaddr[0]
                                if re.search(ip_ignore,ipaddr):
                                        logger.info('%s is in IP ignore list.', ipaddr)
                                else:
                                    try:
                                        ipaddress.ip_address(ipaddr).is_global
                                        scan_ipaddr(ipaddr, apikey)
                                    except:
                                        pass # Ok to pass here, as this captures 
                                                # the odd string which is not an IP Address
                            
                        # Check for URLs and send for report to Virus Total
                        if urls:
                            url = re.search("(?P<url>https?://[^\s]+)", line) or re.search("(?P<url>www[^\s]+)", line)
                            if url:
                                url = url.group("url")
                                if re.search(url_ignore, url):
                                    logger.info('%s is in URL ignore list.', url)
                                else:
                                    scan_url(url, apikey)


                        # Perform search within text files
                        for key, value in flag_list.items():
                            regex = value['regex']
                            desc = value['desc']
                            if re.search(regex, line) and not re.search(
                                    ignore_list, line):
                                logger.error('File contains violation: %s', patch_file)
                                logger.error('Flagged Content: %s', line.rstrip())
                                logger.error('Rationale: %s', desc.rstrip())
                                failure = True
                                with open(reports_dir + "contents_" + project + ".log",
                                        "a") as gate_report:
                                    gate_report.write('File contains violation: {0}\n'.
                                                    format(patch_file))
                                    gate_report.write('Flagged Content: {0}'.
                                                    format(line))
                                    gate_report.write('Matched Regular Exp: {0}\n'.
                                                    format(regex))
                                    gate_report.write('Rationale: {0}\n'.
                                                    format(desc.rstrip()))

def scan_binary(patch_file, project, sha256hash, apikey):
    """
    Sends Binary (sha256hash) to Virus Total API
    """
    v_api = virus_total.VirusTotal()

    while True:
        binary_report = v_api.binary_report(sha256hash, apikey)
        response_code = binary_report['response_code']
    
        # report does not exist, need to scan
        if response_code == 0:
            logger.info('Performing new scan of %s.', patch_file)
            scan_file = v_api.scan_file(patch_file, apikey)
            logger.info('VirusTotal Response: %s', scan_file['verbose_msg'])
            logger.info('Report will be rendered at: %s', scan_file['permalink'])
            binary_report = v_api.binary_report(sha256hash, apikey)

        # Item is still queued
        if response_code == -2:
            logger.info('Report job still queued..')

        if response_code == 1:
            logger.info('Report found, job complete.')
            break

    positives = binary_report['positives']

    if positives == 0:
        negative_report(binary_report,sha256hash, project, patch_file)
    else:
        positive_report(binary_report,sha256hash, project, patch_file)

def negative_report(binary_report,sha256hash, project, patch_file):
    """
    If no match is made and file is clean
    """
    report_url = binary_report['permalink']
    scan_date = binary_report['scan_date'] # build conditional
    logger.info('File scan date for %s shows a clean status on: %s', patch_file, scan_date)
    logger.info('Full report avaliable here: %s', report_url)
    logger.info('The following sha256 hash can be used in your %s.yaml file to suppress this scan:', project)
    logger.info('%s:', sha256hash)
    with open(reports_dir + "binaries-" + project + ".log",
                    "a") as gate_report:
                gate_report.write('Non Whitelisted Binary: {}\n'.
                                    format(patch_file))
                gate_report.write('File scan date for {} shows a clean status on {}\n'.
                                    format(patch_file, scan_date))
                gate_report.write('The following sha256 hash can be used in your {}.yaml file to suppress this scan:\n'.
                                    format(project))
                gate_report.write('{}\n'.
                                    format(sha256hash))

def positive_report(binary_report,sha256hash, project, patch_file):
    """
    If a Positive match is found
    """ 
    failure = True
    report_url = binary_report['permalink']
    scan_date = binary_report['scan_date'] # build conditional
    logger.error("Virus Found!")
    logger.info('File scan date for %s shows a infected status on: %s', patch_file, scan_date)
    logger.info('Full report avaliable here: %s', report_url)

def scan_ipaddr(ipaddr, apikey):
    """
    If an IP Address is found, scan it
    """
    logger.info('Query VirusTotal API for Public IP Found: %s', ipaddr)
    v_api = virus_total.VirusTotal()
    scan_ip = v_api.send_ip(ipaddr, apikey)
    response_code = scan_ip['response_code']
    verbose_msg = scan_ip['verbose_msg']
    urls = scan_ip['detected_urls']

    if urls:
        failure = True
        logger.error('%s has been known to resolve to malicious urls', ipaddr)
        for url in urls:
            logger.info('URL: %s',url)
    else:
        logger.info('%s has no record of resolving to malicious urls', ipaddr)

def scan_url(url, apikey):
    """
    If URL is found, scan it
    """
    logger.info('Found what I believe is a URL: %s', url)
    v_api = virus_total.VirusTotal()
    while True:
        url_report = v_api.url_report(url, apikey)
        response_code = url_report['response_code']

        # report does not exist, need to scan
        if response_code == -2:
            logger.info('Report job still queued..')

        if response_code == 0:
            logger.info('No report  for %s', url)
            break

        if response_code == 1:
            logger.info('Report found, job complete for %s.', url)
            break

    try:
        positives = url_report['positives']
        if positives > 0:
            for site, results in url_report['scans'].items():
                if results['detected']:
                    detected = True
                    failure = True
                    logger.error("%s is recorded as a %s by %s", url, results['result'], site)
            if detected:
                logger.error("Full report available here: %s", url_report['permalink'])
        else:
            logger.info("%s is recorded as a clean", url)
    except:
        pass
    
def process_failure():
    """
    If any scan operations register a failure, sys.exit(1) is called
    to allow build to register a failure
    """
    if failure:
        sys.exit(1)



import json
import logging
import os
import re
import requests
import redis
import sys
import time
import urllib
import uuid
from pylimit import PyRateLimit
import six.moves.configparser

class VirusTotal():
    def __init__(self, *args):
        self.logger = logging.getLogger(__name__)
        self.base_url = 'https://www.virustotal.com/vtapi/v2/'
        self.HTTP_OK = 200
        self.HTTP_BAD = 400
        self.HTTP_FORBIDDEN = 403
        self.HTTP_RATE_EXCEEDED = 204
        self.public_api_sleep_time = 20
        self.logger = logging.getLogger(__name__)
        self.uuid = uuid.uuid4()
        self.config = six.moves.configparser.SafeConfigParser()
        self.config.read('anteater.conf')

        try: # might be better to move this to its own class
            conn = redis.StrictRedis(
                host='localhost',
                port=6379,
                password='')
            conn.ping()
            PyRateLimit.init(redis_host="localhost", redis_port=6379)
        except Exception as ex:
            self.logger.error('Error: %s', ex)
            exit('Failed to connect, terminating.')
        
        self.limit = PyRateLimit()

        try:
            vt_rate_type = self.config.get('config', 'vt_rate_type')
        except six.moves.configparser.NoSectionError:
            self.logger.error("A config section is required for vt_rate_type with a public | private option ")
            sys.exit(1)

        patten = re.compile(r'\bpublic\b|\bprivate\b')
        if not patten.match(vt_rate_type):
            self.logger.error("Unrecognized %s option for vt_rate_type", vt_rate_type)
            sys.exit(1)

        if vt_rate_type == 'public':
            self.limit.create(21,1)
        elif vt_rate_type == 'private':
            self.limit.create(1,1)

    def rate_limit(self):
        """
        Simple rate limit function using redis
        """
        rate_limited_msg = False

        while True:
            is_rate_limited = self.limit.is_rate_limited(uuid)
            if is_rate_limited:
                time.sleep(0.3) # save hammering the shit out of redis
                if not rate_limited_msg:
                    self.logger.info('Rate limit active..please wait...')
                    rate_limited_msg = True
                    
            
            if not is_rate_limited:
                self.logger.info('Rate limit clear.')
                self.limit.attempt(uuid)
                return True

    def scan_file(self, filename, apikey):
        """
        Sends a file to virus total for assessment
        """
        url = self.base_url + "file/scan"
        params = {'apikey': apikey}
        scanfile = {"file": open(filename, 'rb')}
        response = requests.post(url, files=scanfile, params=params)
        rate_limit_clear = self.rate_limit()
        if rate_limit_clear:
            if response.status_code == self.HTTP_OK:
                json_response = response.json()
                return json_response
            elif response.status_code == self.HTTP_RATE_EXCEEDED:
                time.sleep(20)
            else:
                self.logger.error("sent: %s, HTTP: %d", filename, response.status_code)

    def rescan_file(self, filename, sha256hash, apikey):
        """
        just send the hash, check the date
        """
        url = self.base_url + "file/rescan"
        params = {
            'apikey': apikey,
            'resource': sha256hash
        }
        
        headers = {
            "Accept-Encoding": "gzip, deflate",
            "User-Agent" : "gzip,  lhinds" # set this to a config value!
        }
        rate_limit_clear = self.rate_limit()
        if rate_limit_clear:
            response = requests.post(url, params=params)
            if response.status_code == self.HTTP_OK:
                self.logger.info("sent: %s, HTTP: %d, content: %s", os.path.basename(filename), response.status_code, response.text)
            elif response.status_code == self.HTTP_RATE_EXCEEDED:
                    time.sleep(20)
            else:
                self.logger.error("sent: %s, HTTP: %d", os.path.basename(filename), response.status_code)
            return response

    def binary_report(self, sha256sum, apikey):
        """
        retrieve report from file scan
        """
        url = self.base_url + "file/report"
        params = {"apikey": apikey, "resource": sha256sum}
        rate_limit_clear = self.rate_limit()

        if rate_limit_clear:
            response = requests.post(url, data=params)
            
            if response.status_code == self.HTTP_OK:
                json_response = response.json()
                response_code = json_response['response_code']
                return json_response
            elif response.status_code == self.HTTP_RATE_EXCEEDED:
                time.sleep(20)
            else:
                self.logger.warning("retrieve report: %s, HTTP code: %d", os.path.basename(filename), response.status_code)
        

    def send_ip(self, ipaddr, apikey):
        """
        Send IP address for list of past malicous domain associations
        """
        url = self.base_url + "ip-address/report"
        parameters = {"ip": ipaddr, "apikey": apikey}
        rate_limit_clear = self.rate_limit()
        if rate_limit_clear:
            response = requests.get(url, params=parameters)
            if response.status_code == self.HTTP_OK:
                json_response = response.json()
                return json_response
            elif response.status_code == self.HTTP_RATE_EXCEEDED:
                    time.sleep(20)
            else:
                self.logger.error("sent: %s, HTTP: %d", ipaddr, response.status_code)
            time.sleep(self.public_api_sleep_time)

    def url_report(self, scan_url, apikey):
        """
        Send URLS for list of past malicous associations
        """
        headers = {
            "Accept-Encoding": "gzip, deflate",
            "User-Agent" : "gzip,  My Python requests library example client or username"
            }
        url = self.base_url + "url/report"
        params = {"apikey": apikey, 'resource':scan_url}
        rate_limit_clear = self.rate_limit()
        if rate_limit_clear:
            response = requests.post(url, params=params, headers=headers)
            if response.status_code == self.HTTP_OK:
                json_response = response.json()
                return json_response
            elif response.status_code == self.HTTP_RATE_EXCEEDED:
                time.sleep(20)    
            else:
                self.logger.error("sent: %s, HTTP: %d", scan_url, response.status_code)
            time.sleep(self.public_api_sleep_time)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2017 Luke Hinds <lukehinds@gmail.com>
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0
##############################################################################

"""
    Accepts the --path argument and iterates the root directory using os.walk
    If a file is a binary, or contains a blacklisted string. If any violations
    are found, the script adds the violation to a log file.
"""

from __future__ import division, print_function, absolute_import
import ipaddress
import logging
import hashlib
import json
import six.moves.configparser
import os
import re
import sys
import time
from binaryornot.check import is_binary
from . import get_lists
from . import virus_total

logger = logging.getLogger(__name__)
config = six.moves.configparser.SafeConfigParser()
config.read('anteater.conf')
anteater_files = config.get('config', 'anteater_files')
reports_dir = config.get('config', 'reports_dir')
ignore_dirs = ['.git', 'examples', anteater_files]
hasher = hashlib.sha256()

def prepare_project(project, project_dir, bincheck, ips, urls):
    """
    Generates blacklists / whitelists and calls main functions
    """

    # Get Various Lists / Project Waivers
    lists = get_lists.GetLists()

    # Get file name black list and project waivers
    file_audit_list, file_audit_project_list = lists.file_audit_list(project)

    # Get file content black list and project waivers
    flag_list, ignore_list = lists.file_content_list(project)

    # Get File Ignore Lists
    file_ignore = lists.file_ignore()

    ignore_directories = lists.ignore_directories(project)

    # Get URL Ignore Lists
    url_ignore = lists.url_ignore(project)

    # Get URL Ignore Lists
    ip_ignore = lists.ip_ignore(project)

    hashlist = get_lists.GetLists()

    if bincheck or ips or urls:
        try:  
            apikey = os.environ["VT_KEY"]
        except KeyError: 
            logger.error("Please set your virustotal.com API key as an environment variable")
            sys.exit(1)
        try:
            vt_rate_type = config.get('config', 'vt_rate_type')
        except six.moves.configparser.NoSectionError:
            logger.error("A config section is required for vt_rate_type with a public | private option ")
            sys.exit(1)

        patten = re.compile(r'\bpublic\b|\bprivate\b')
        if not patten.match(vt_rate_type):
            logger.error("Unrecognized %s option for vt_rate_type", vt_rate_type)
            sys.exit(1)

    # Perform rudimentary scans
    scan_file(project, project_dir, bincheck, ips, urls, file_audit_list,
              file_audit_project_list, flag_list, ignore_list, hashlist, 
              file_ignore, ignore_directories, url_ignore, ip_ignore, apikey)


def scan_file(project, project_dir, bincheck, ips, urls, file_audit_list,
              file_audit_project_list, flag_list, ignore_list, hashlist,
              file_ignore, ignore_directories, url_ignore, ip_ignore, apikey):

    """
    Main scan tasks begin
    """
    logger.info("Commencing scan tasks..")
    for root, dirs, files in os.walk(project_dir):
        # Filter out ignored directories from list.
        for items in files:
            full_path = os.path.join(root, items)
            split_path = full_path.split(project + '/', 1)[-1]
            # Check for Blacklisted file names
            if not any(x in split_path for x in ignore_directories):
                if file_audit_list.search(full_path) and not \
                        file_audit_project_list.search(full_path):
                    match = file_audit_list.search(full_path)
                    logger.error('Blacklisted filename: %s', full_path)
                    logger.error('Matched String: %s', match.group())
                    with open(reports_dir + "file-names_" + project + ".log",
                              "a") as gate_report:
                                gate_report. \
                                    write('Blacklisted filename: {0}\n'.
                                          format(full_path))
                                gate_report. \
                                    write('Matched String: {0}'.
                                          format(match.group()))

                # Check if Binary is whitelisted
                if is_binary(full_path) and bincheck:
                    
                    split_path = full_path.split(project + '/', 1)[-1]
                    binary_hash = hashlist.binary_hash(project, split_path)
                    with open(full_path, 'rb') as afile:
                        hasher = hashlib.sha256()
                        buf = afile.read()
                        hasher.update(buf)
                        sha256hash = hasher.hexdigest()
                    if sha256hash in binary_hash:
                        logger.info('Found matching file hash for: %s',
                                            full_path)
                        logger.info('No further action needed for: %s',
                                            full_path)
                    else:
                        logger.info('Non Whitelisted Binary file: %s', full_path)
                        scan_binary(full_path, split_path, project, sha256hash, apikey)                        
                        
                else:
                    if not items.endswith(tuple(file_ignore)) and not is_binary(full_path):
                        try:
                            fo = open(full_path, 'r')
                            lines = fo.readlines()
                        except IOError:
                            logger.error('%s does not exist', full_path)

                        for line in lines:
                            # Find IP Addresses and send for report to Virus Total
                            if ips:
                                ipaddr = re.findall(r'(?:\d{1,3}\.)+(?:\d{1,3})', line)
                                if ipaddr:
                                    ipaddr = ipaddr[0]
                                    if re.search(ip_ignore,ipaddr):
                                        logger.info('%s is in IP ignore list.', ipaddr)
                                    else:
                                        try:
                                            ipaddress.ip_address(ipaddr).is_global
                                            scan_ipaddr(ipaddr, project, split_path, apikey)
                                        except:
                                            pass # Ok to pass here, as this captures 
                                                # the odd string which is not an IP Address
                            
                            # Check for URLs and send for report to Virus Total
                            if urls:
                                url = re.search("(?P<url>https?://[^\s]+)", line) or re.search("(?P<url>www[^\s]+)", line)
                                if url:
                                    url = url.group("url")
                                    if re.search(url_ignore, url):
                                        logger.info('%s is in URL ignore list.', url)
                                    else:
                                         scan_url(url, project, split_path, apikey)

                            # Check flagged content in files
                                
                            for key, value in flag_list.items():
                                regex = value['regex']
                                desc = value['desc']
                                if re.search(regex, line) and not re.search(
                                        ignore_list, line):
                                    logger.error('File contains violation: %s',
                                                full_path)
                                    logger.error('Flagged Content: %s',
                                                line.rstrip())
                                    logger.error('Matched Regular Exp: %s', regex)
                                    logger.error('Rationale: %s', desc.rstrip())
                                    with open(reports_dir + "contents-" + project
                                                    + ".log", "a") \
                                            as gate_report:
                                        gate_report. \
                                            write('File contains violation: {0}\n'.
                                                format(full_path))
                                        gate_report. \
                                            write('Flagged Content: {0}'.
                                                format(line))
                                        gate_report. \
                                            write('Matched Regular Exp: {0}'.
                                                format(regex))
                                        gate_report. \
                                            write('Rationale: {0}\n'.
                                                format(desc.rstrip()))

def scan_binary(full_path, split_path, project, sha256hash, apikey):
    """
    Sends Binary (sha256hash) to Virus Total API
    """
    v_api = virus_total.VirusTotal()

    while True:
        binary_report = v_api.binary_report(sha256hash, apikey)
        response_code = binary_report['response_code']
    
        # report does not exist, need to scan
        if response_code == 0:
            logger.info('Performing new scan of %s.', split_path)
            scan_file = v_api.scan_file(full_path, apikey)
            logger.info('VirusTotal Response: %s', scan_file['verbose_msg'])
            logger.info('Report will be rendered at: %s', scan_file['permalink'])
            binary_report = v_api.binary_report(sha256hash, apikey)

        # Item is still queued
        if response_code == -2:
            logger.info('Report job still queued..')

        if response_code == 1:
            logger.info('Report found, job complete.')
            break

    positives = binary_report['positives']

    if positives == 0:
        negative_report(binary_report,sha256hash, split_path, project, full_path)
    else:
        positive_report(binary_report,sha256hash, split_path, project, full_path)

def negative_report(binary_report,sha256hash, split_path, project, full_path):
    """
    If no match is made and file is clean
    """
    report_url = binary_report['permalink']
    scan_date = binary_report['scan_date'] # build conditional
    logger.info('File scan date for %s shows a clean status on: %s', split_path, scan_date)
    logger.info('Full report avaliable here: %s', report_url)
    logger.info('The following sha256 hash can be used in your %s.yaml file to suppress this scan:', project)
    logger.info('%s:', sha256hash)
    with open(reports_dir + "binaries-" + project + ".log",
                    "a") as gate_report:
                gate_report.write('Non Whitelisted Binary: {}\n'.
                                    format(full_path))
                gate_report.write('File scan date for {} shows a clean status on {}\n'.
                                    format(split_path, scan_date))
                gate_report.write('The following sha256 hash can be used in your {}.yaml file to suppress this scan:\n'.
                                    format(project))
                gate_report.write('{}\n'.
                                    format(sha256hash))

def positive_report(binary_report,sha256hash, split_path, project, full_path):
    """
    If a Positive match is found
    """ 
    report_url = binary_report['permalink']
    scan_date = binary_report['scan_date'] # build conditional
    logger.error("Virus Found!")
    logger.info('File scan date for %s shows a infected status on: %s', split_path, scan_date)
    logger.info('Full report avaliable here: %s', report_url)
    with open(reports_dir + "binaries-" + project + ".log",
                    "a") as gate_report:
                gate_report.write('Virus Found!: {}\n'.
                                    format(full_path))
                gate_report.write('File scan date for {} shows a infected status on: {}\n'.
                                    format(split_path, scan_date))
                gate_report.write('Full report avaliable here: {}\n'.
                                    format(report_url))

def scan_ipaddr(ipaddr, project, split_path, apikey):
    """
    If an IP Address is found, scan it
    """
    logger.info('File %s contains what I believe is a IP Address: %s', split_path, ipaddr)
    with open(reports_dir + "ips-" + project + ".log", "a") as gate_report:
        gate_report.write('File {} contains what I believe is an IP Address: {}\n'.format(split_path, ipaddr))
    v_api = virus_total.VirusTotal()
    scan_ip = v_api.send_ip(ipaddr, apikey)
    response_code = scan_ip['response_code']
    verbose_msg = scan_ip['verbose_msg']
    urls = scan_ip['detected_urls']
    with open(reports_dir + "ips-" + project + ".log",
                    "a") as gate_report:
        if urls:
            logger.error('%s has been known to resolve to malicious urls', ipaddr)
            gate_report.write('{} has been known to resolve to malicious urls: \n'.
                                        format(ipaddr))
            for url in urls:
                #logger.error('URL: %s',url)
                logger.error('URL: %s',url['url'])
                gate_report.write('{}\n'.format(url['url']))
        else:
            logger.info('No malicious DNS history found for: %s',ipaddr)
            gate_report.write('No malicious DNS history found for: {}\n'.format(ipaddr))

def scan_url(url, project, split_path, apikey):
    """
    If URL is found, scan it
    """
    logger.info('File %s contains what I believe is a URL: %s', split_path, url)
    with open(reports_dir + "urls-" + project + ".log", "a") as gate_report:
        gate_report.write('File {} contains what I believe is a URL: {}\n'.format(split_path, url))

    v_api = virus_total.VirusTotal()
    while True:
        url_report = v_api.url_report(url, apikey)
        response_code = url_report['response_code']

        # report does not exist, need to scan
        if response_code == -2:
            logger.info('Report job still queued..')

        if response_code == 0:
            logger.info('No report  for %s', url)
            break

        if response_code == 1:
            logger.info('Report found, job complete for %s.', url)
            break

    try:
        positives = url_report['positives']
        if positives > 0:
            for site, results in url_report['scans'].items():
                printed = False
                if results['detected']:
                    with open(reports_dir + "urls-" + project + ".log", "a") as gate_report:
                        logger.error("%s is recorded as a %s by %s", url, results['result'], site)
                        gate_report.write('{} is recorded as a {} by {}\n'.format(url, results['result'], site))
                        
            if not printed:
                printed = True
                with open(reports_dir + "urls-" + project + ".log", "a") as gate_report:
                    logger.error("Full report available here: %s", url_report['permalink'])
                    gate_report.write('Full report available here: {}\n'.format(url_report['permalink']))

        else:
            logger.info("%s is recorded as a clean", url)
            with open(reports_dir + "urls-" + project + ".log", "a") as gate_report:
                gate_report.write('{} is recorded as a clean\n'.format(url))

    except:
        pass
